#include <xs1.h>
#include <print.h>
#include "port32A.h"
#include "devicedefines.h"

#define PORT32A_PEEK(X) {asm("peek %0, res[%1]":"=r"(X):"r"(XS1_PORT_32A));} 
#define PORT32A_OUT(X)  {asm("out res[%0], %1"::"r"(XS1_PORT_32A),"r"(X));}

//:codec_init
void CodecInit(chanend ?c_codec) 
{
    return;
}
//:

//:codec_config
/* Configures the CODEC for the required sample frequency.  
 * CODEC reset and frequency select are connected to port 32A
 *
 * Port 32A is shared with other functionality (LEDs etc) so we
 * access via inline assembly. We also take care to retain the
 * state of the other bits.
 */

#define DELAY 20000

void delay(long delay){
    timer t;
    unsigned time;

    t :> time;
    time += delay;
    t when timerafter(time) :> int _;
}

void bluestring_SPI(unsigned int word){
    unsigned int x;
    PORT32A_PEEK(x);
    x &= (~SPI_CS);
    x |= P32A_USB_RST;
    PORT32A_OUT(x);

   // delay(DELAY);

    for (int i=7; i>=0; i--)
    {
        PORT32A_PEEK(x);
        x &= (~SPI_SCK); //SCK low
        PORT32A_OUT(x);

        if (word & 1<<i){
            PORT32A_PEEK(x);
            x |= SPI_DATA;
            PORT32A_OUT(x);
        }

        else{
            PORT32A_PEEK(x);
            x &= (~SPI_DATA);
            PORT32A_OUT(x);
        }

        delay(DELAY);

        PORT32A_PEEK(x);
        x |= SPI_SCK;
        PORT32A_OUT(x);

        delay(DELAY);
    }

    delay(DELAY);
    PORT32A_PEEK(x);
    x |= SPI_CS; //CS high
    PORT32A_OUT(x);
    delay(DELAY);
}


void bluestring_setSampleFreq(unsigned int samFreq){
    int x;
    asm("peek %0, res[%1]":"=r"(x):"r"(XS1_PORT_32A));

    switch(samFreq)
    {
    case (44100) :
        bluestring_SPI(44);
        x |= LED_441;
        x &= (~LED_48);
        x &= (~LED_882);
        x &= (~LED_96);
        x &= (~LED_1764);
        x &= (~LED_192);
        break;

    case (48000) :
        bluestring_SPI(48);
        x |= LED_48;
        x &= (~LED_441);
        x &= (~LED_882);
        x &= (~LED_96);
        x &= (~LED_1764);
        x &= (~LED_192);
        break;

    case (88200) :
        bluestring_SPI(88);
        x |= LED_882;
        x &= (~LED_441);
        x &= (~LED_48);
        x &= (~LED_96);
        x &= (~LED_1764);
        x &= (~LED_192);
        break;

    case (96000) :
        bluestring_SPI(96);
        x |= LED_96;
        x &= (~LED_441);
        x &= (~LED_48);
        x &= (~LED_882);
        x &= (~LED_1764);
        x &= (~LED_192);
        break;

    case (176400) :
        bluestring_SPI(176);
        x |= LED_1764;
        x &= (~LED_441);
        x &= (~LED_48);
        x &= (~LED_882);
        x &= (~LED_96);
        x &= (~LED_192);
        break;

    case (192000) :
        bluestring_SPI(192);
        x |= LED_192;
        x &= (~LED_441);
        x &= (~LED_48);
        x &= (~LED_882);
        x &= (~LED_96);
        x &= (~LED_1764);
        break;
    }
    asm("out res[%0], %1"::"r"(XS1_PORT_32A),"r"(x));
}

void CodecConfig(unsigned samFreq, unsigned mClk, chanend ?c_codec)
{
    timer t;
    unsigned time;

    /* Put CODEC reset line low */
    bluestring_SPI(0xff);


    bluestring_setSampleFreq(samFreq);

    //Hold in reset for 2ms
    t :> time;
    time += 200000;
    t when timerafter(time) :> int _;

    /* Codec out of reset */
    bluestring_SPI(200);
}
//:
