#include "coProcessor.h"
#include "iic.h"
#include "global.h"

/*
 * Apple Authentication Coprocessor Functions
 * Ref 2.0B Specification
 */

int coProcessorAddress = COPRO_IIC_ADDRESS;
#ifdef CS4351
int dacAddress = DAC_IIC_ADDRESS;
#endif

int coProcessorInit(timer t, chanend c, port p_scl, port p_sda)
{
  unsigned char retVal[1];

  iic_phy_wait((50000 * XS1_TIMER_MHZ), t); // 50ms

#ifdef CS4351
#error TIDY THIS UP
  if (iic_readC(dacAddress, 0x1, retVal, 1,c))
    return 1;

  retVal[0] = 0x40;
  if (iic_writeC(dacAddress, 0x8, retVal, 1,c))
    return 1;

  retVal[0] = 0x1 << 4;
  if (iic_writeC(dacAddress, 0x2, retVal, 1,c))
    return 1;
#endif

  if (iic_readC(coProcessorAddress, Register_DeviceVersion, retVal, 1, c, p_scl, p_sda))
    return 1;
  if (retVal[0] != COPROCESSOR_VERSION)
    return 1;

  if (iic_readC(coProcessorAddress, Register_FirmwareVersion, retVal, 1, c, p_scl, p_sda))
     return 1;
  if (retVal[0] != COPROCESSOR_FW_VERSION)
    return 1;

  return 0;
}

int coProcessorGetAuthVersion(char *major, char *minor, chanend c, port p_scl, port p_sda)
{
  if (iic_readC(coProcessorAddress, Register_AuthProtocolMajorVersion, (unsigned char *)major, 1, c, p_scl, p_sda))
    return 1;
  if (iic_readC(coProcessorAddress, Register_AuthProtocolMinorVersion, (unsigned char *)minor, 1, c, p_scl, p_sda))
    return 1;

  return 0;

}

int coProcessorGetDeviceID(int *retVal, chanend c, port p_scl, port p_sda)
{
  if (iic_readC(coProcessorAddress, Register_DeviceID, (unsigned char *)retVal, 4, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorGetAccessoryCertificateLength(unsigned short *len, chanend c, port p_scl, port p_sda)
{
    unsigned char val[2];

    if (iic_readC(coProcessorAddress, Register_AccessoryCertDataLen, val, 2, c, p_scl, p_sda))
    {
        return 1;
    }
    
    *len = val[1] | (val[0] << 8);

    return 0;
}

int coProcessorGetAccessoryCertificatePage(unsigned short len, unsigned char *data, unsigned page, chanend c, port p_scl, port p_sda)
{
    if (iic_readC(coProcessorAddress, Register_AccessoryCertDataPtr + page, data, len, c, p_scl, p_sda))
    {
        return 1;
    }

    return 0;
}

int coProcessorGetAccessoryCertificate(unsigned short *len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  unsigned char val[2];

  if (iic_readC(coProcessorAddress, Register_AccessoryCertDataLen, val, 2, c, p_scl, p_sda))
    return 1;

  *len = val[1] | (val[0] << 8);

  if (*len > 1920)
    return 1;

  if (iic_readC(coProcessorAddress, Register_AccessoryCertDataPtr, data, *len, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorGetSignature(unsigned *len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  unsigned char lenarray[2];

  if (iic_readC(coProcessorAddress, Register_SigDataLength, lenarray, 2, c, p_scl, p_sda))
    return 3;

  *len = (lenarray[0] << 8) | lenarray[1];

  if (*len > 128)
    return 4;

  if (*len == 0)
    return 5;

//  while(coProcessorBusy(c)) {}

  if (iic_readC(coProcessorAddress, Register_SigData, data, *len, c, p_scl, p_sda))
    return 6;

  return 0;
}

int coProcessorGetChallenge(unsigned short *len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  if (iic_readC(coProcessorAddress, Register_ChallDataLength, (unsigned char *)len, 2, c, p_scl, p_sda))
    return 1;

  if (*len > 20)
    return 1;

  if (iic_readC(coProcessorAddress, Register_ChallData, data, *len, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorPutChallenge(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  char lenarray[2];
  if (len > 20)
    return 1;

  lenarray[0] = (len >> 8) & 0xFF;
  lenarray[1] = (len & 0xFF);

  if (iic_writeC(coProcessorAddress, Register_ChallDataLength, (unsigned char*)lenarray, 2, c, p_scl, p_sda))
    return 2;

  // Busy wait loop, necessary at 50kHz i2c
  while(coProcessorBusy(c, p_scl, p_sda)) {}

  if (iic_writeC(coProcessorAddress, Register_ChallData, data, len, c, p_scl, p_sda))
    return 3;
  return 0;
}

int coProcessorStartSigGeneration(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal = ProcControl_StartSigGeneration;

  if (iic_writeC(coProcessorAddress, Register_AuthControlStatus, &retVal, 1, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorStartSigVerification(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal = ProcControl_StartSigVerification;

  if (iic_writeC(coProcessorAddress, Register_AuthControlStatus, &retVal, 1, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorPutSignature(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  if (iic_writeC(coProcessorAddress, Register_SigDataLength, (unsigned char *)&len, 2, c, p_scl, p_sda))
    return 1;

  if (len > 20)
    return 1;

  if (iic_writeC(coProcessorAddress, Register_SigData, data, len, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorBusy(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal;;

  if (iic_readC(coProcessorAddress, Register_DeviceVersion, &retVal, 1, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorValidResults(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal;

  if (iic_readC(coProcessorAddress, Register_AuthControlStatus, &retVal, 1, c, p_scl, p_sda))
    return 1;

  // Check PROC_RESULTS for non-zero value (indicates success)
  if ((retVal >> 4) & 0x7)
  {
    return 0;
  }
  else
  {
    return 1;
  }

  return 0;
}


int coProcessorGenerateChallenge(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal = ProcControl_StartChallGeneration;

  if (iic_writeC(coProcessorAddress, Register_AuthControlStatus, &retVal, 1, c, p_scl, p_sda))
    return 1;

  return 0;
}

int coProcessorPutiPodCert(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda)
{
  int i;

  if (iic_writeC(coProcessorAddress, Register_iPodCertDataLen, (unsigned char *)&len, 2, c, p_scl, p_sda))
    return 1;

  if (len > 1024)
    return 1;

  for (i = 0; i < len / 128; i++)
  {
    if (iic_writeC(coProcessorAddress, Register_iPodCertDataPtr + i, data, len, c, p_scl, p_sda))
      return 1;

    data += 128;
  }

  return 0;
}

int coProcessorStartCertValidation(chanend c, port p_scl, port p_sda)
{
  unsigned char retVal = ProcControl_StartCertValidation;

  if (iic_writeC(coProcessorAddress, Register_AuthControlStatus, &retVal, 1, c, p_scl, p_sda))
    return 1;

  return 0;
}
