#ifndef COPROCESSOR_H_
#define COPROCESSOR_H_

#include <xccompat.h>
#include "coProcessor_registers.h"

/*
 * Apple Authentication Coprocessor Functions
 * Ref 2.0B Specification
 */

// ---------------------------------------------------------------

/*
 * coProcessorInit
 * Verify coprocessor device version
 */
#if defined __XC__
int coProcessorInit(timer t, chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorInit(timer t, chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorBusy
 * Checks if coProcessor is still running previous command
 */
#if defined __XC__
int coProcessorBusy(chanend ?c, port ?p_scl, port ?p_sda);

#elif defined __STDC__
int coProcessorBusy(chanend c, port p_scl, port p_sda);

#endif

/*
 * coProcessorValidResults
 * Check if previous command was successful
 */
#if defined __XC__
int coProcessorValidResults(chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorValidResults(chanend c, port p_scl, port p_sda);
#endif


/*
 * ---------- CoProcessor commands ----------
 */

/*
 * coProcessorStartSigGeneration
 * Start signature generation from a iPod challenge
 */
#if defined __XC__
int coProcessorStartSigGeneration(chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorStartSigGeneration(chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorStartSigVerification
 * Start validating an iPod's response
 */
#if defined __XC__
int coProcessorStartSigVerification(chanend ?c);
#elif defined __STDC__
#endif

/*
 * coProcessorGenerateChallenge
 * Generate a challenge for the iPod
 */
#if defined __XC__
int coProcessorGenerateChallenge(chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGenerateChallenge(chanend c, port p_scl, port p_sda);
#endif


/*
 * coProcessorStartCertValidation
 * Validate iPod's certificate
 */
#if defined __XC__
int coProcessorStartCertValidation(chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorStartCertValidation(chanend c, port p_scl, port p_sda);
#endif

/*
 * ---------- CoProcessor data retrieval ----------
 */

#if defined __XC__
int coProcessorGetAuthVersion(char &major, char &minor, chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetAuthVersion(char *major, char *minor, chanend c, port p_scl, port p_sda);
#endif


/*
 * coProcessorGetDeviceID
 * Retrieve Device ID of Coprocessor
 */
#if defined __XC__
int coProcessorGetDeviceID(int &retVal, chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetDeviceID(int *retVal, chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorGetAccessoryCertificate
 * Retrieve Accessory Certificate stored in coProcessor flash
 */
#if defined __XC__
int coProcessorGetAccessoryCertificate(unsigned short &len, unsigned char data[], chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetAccessoryCertificate(unsigned short *len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

#if defined __XC__
int coProcessorGetAccessoryCertificatePage(unsigned short len, unsigned char data[], unsigned page, chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetAccessoryCertificatePage(unsigned short len, unsigned char *data, unsigned page, chanend c, port p_scl, port p_sda);
#endif

/* 
 * coProcessorGetAccessoryCertificateLength
 * Retried Accessory Certificate from the coProcessor 
 */
#if defined __XC__
int coProcessorGetAccessoryCertificateLength(unsigned short &len, chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetAccessoryCertificateLength(unsigned short *len, chanend c,  port p_scl, port p_sda);
#endif

/*
 * coProcessorGetSignature
 * Retrieve signature generated from challenge
 */
#if defined __XC__
int coProcessorGetSignature(unsigned &len, unsigned char data[], chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetSignature(unsigned *len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorGetChallenge
 * Retrieve challenge generated by coProcessor
 */
#if defined __XC__
int coProcessorGetChallenge(unsigned short &len, unsigned char data[], chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorGetChallenge(unsigned short *len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

/*
 * ---------- CoProcessor data storing ----------
 */

/*
 * coProcessorPutChallenge
 * Store challenge generated by iPod
 */
#if defined __XC__
int coProcessorPutChallenge(unsigned short len, unsigned char data[], chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorPutChallenge(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorPutSignature
 * Store signature generated by iPod
 */
#if defined __XC__
int coProcessorPutSignature(unsigned short len, unsigned char data[], chanend ?c, port ?p_scl, port ?p_sda);
#elif defined __STDC__
int coProcessorPutSignature(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

/*
 * coProcessorPutiPodCert
 * Store iPod's certificate for verification
 */
#if defined __XC__
int coProcessorPutiPodCert(unsigned short len, unsigned char data[], chanend ?c, port ?p_scl, port p_sda);
#elif defined __STDC__
int coProcessorPutiPodCert(unsigned short len, unsigned char *data, chanend c, port p_scl, port p_sda);
#endif

#endif /*COPROCESSOR_H_*/
