#ifndef IAP_H
#define IAP_H
#define MAX_IAP_PACKET_SIZE 256
#define IAP_USB_BUFFER_TO_HOST_SIZE (256)
#define IAP_ACK 20
#define IAP_RESET 30
#define IAP_BUFFER_SIZE 139 // This should be just enough to hold a maximum size RetAccessoryAuthenticationInfo (plus one for checksum when using transaction id as well) but this exceeds wMaxPacketSize

#define MS XS1_TIMER_KHZ // Millisecond

#ifdef __IAP_IMPL
#define INLINE
#else
#define INLINE inline
#endif

#define GENERAL_LINGO 0x0
enum {
  GENERAL_LINGO_RequestIdentify = 0x00,
  GENERAL_LINGO_iPodAck = 0x02,
  GENERAL_LINGO_RequestTransportMaxPayloadSize = 0x11,
  GENERAL_LINGO_ReturnTransportMaxPayloadSize = 0x12,
  GENERAL_LINGO_GetAccessoryAuthenticationInfo = 0x14,
  GENERAL_LINGO_RetAccessoryAuthenticationInfo = 0x15,
  GENERAL_LINGO_AckAccessoryAuthenticationInfo = 0x16,
  GENERAL_LINGO_GetAccessoryAuthenticationSignature = 0x17,
  GENERAL_LINGO_RetAccessoryAuthenticationSignature = 0x18,
  GENERAL_LINGO_AckAccessoryAuthenticationStatus = 0x19,
  GENERAL_LINGO_StartIDPS = 0x38,
  GENERAL_LINGO_SetFIDTokenValues = 0x39,
  GENERAL_LINGO_AckFIDTokenValues = 0x3A,
  GENERAL_LINGO_EndIDPS = 0x3B,
  GENERAL_LINGO_IDPSStatus = 0x3C,
  GENERAL_LINGO_SetEventNotification = 0x49,
  GENERAL_LINGO_GetiPodOptionsForLingo = 0x4B,
  GENERAL_LINGO_RetiPodOptionsForLingo = 0x4C,
  GENERAL_LINGO_SetAvailableCurrent = 0x54,
};

enum GeneralLingo_iPodAck_Status {
  IPOD_ACK_Success = 0x00,
  IPOD_ACK_Bad_parameter = 0x04,
  IPOD_ACK_Maximum_connections_reached = 0x15,
};

#ifdef NO_INLINE_IAP_SELECT_HANDLER
#pragma select handler
void iap_get_ack_or_reset_or_data(chanend c, int &is_ack, int &is_reset, unsigned int &datum);
#else
#pragma select handler
INLINE void iap_get_ack_or_reset_or_data(chanend c, int &is_ack, int &is_reset, unsigned int &datum) {
  if (testct(c)) {
    int ct = inct(c);
    switch(ct) {
       case IAP_ACK:
          is_ack = 1;
          is_reset = 0;
          break;
       case IAP_RESET:
          is_ack = 0;
          is_reset = 1;
          break;
    }
    (void) inuchar(c);
    (void) inuchar(c);
    (void) inuchar(c);
  } else {
    is_ack = 0;
    is_reset = 0;
    datum = inuint(c);
  }
}
#endif

INLINE void iap_send_ack(chanend c) {
  outct(c, IAP_ACK);
  outuchar(c, 0);
  outuchar(c, 0);
  outuchar(c, 0);
}

INLINE void iap_send_reset(chanend c) {
  outct(c, IAP_RESET);
  outuchar(c, 0);
  outuchar(c, 0);
  outuchar(c, 0);
}

void iAP(chanend c_iap, chanend ?c_i2c, port ?p_scl, port ?p_sda);
void init_iAP(chanend ?c_i2c, port ?p_scl, port ?p_sda);
void handle_iap_case(int is_ack, int is_reset, unsigned int datum, chanend c_iap, chanend ?c_i2c, port ?p_scl, port ?p_sda);
int handle_poll_dev_det(timer poll);
void set_iAP_timeout(unsigned int timeout);
void clear_iAP_timeout();
void check_iAP_timeout(chanend c_iap);
#endif
