#include <xs1.h>
#include <stdio.h>
#include <print.h>
#include "iic.h"
#include "iAP.h"
#include "coProcessor.h"
#include "xud.h"
#include "devicedefines.h"
#include "queue.h"
#include "iapuser.h"

const int refFreqMHz = 100;
extern timer iAPTimer;

#define SERIAL_STR "0000"
#warning Using default serial string

#define COPROCESSOR_RETRIES 5

void bufCopy(char src[], char dest[], int srcOffset, int destOffset, int len) 
{
    for (int i = 0; i != len; i++) 
    {
        dest[destOffset + i] = src[srcOffset + i];
    }
}

unsigned char calcChecksum(char buffer[], int startPos, int length) 
{
    char sum = 0;
    for(int i = 0; i != length; i++) 
    {
        sum += buffer[startPos + i];
    }
    return -sum;
}

int AssembleCommand(char commandBuff[], char lingoID, char commandID, unsigned short transactionID, char commandData[], int commandDataLen) {
   int lengthOfPacketPayload = 4 + commandDataLen; // lingoID + commandID + transactionID (2 bytes)
   int dataOffset = 0;
   commandBuff[dataOffset++] = 0x55; // Packet start byte
   if (lengthOfPacketPayload > 255) {
      // Large packet format
      commandBuff[dataOffset++] = 0; // Packet payload length marker
      commandBuff[dataOffset++] = (lengthOfPacketPayload >> 8) & 0xFF; // Packet payload length (bits 15:8)
      commandBuff[dataOffset++] = lengthOfPacketPayload & 0xFF; // Packet payload length (bits 7:0)
   } else {
      // Small packet format
      commandBuff[dataOffset++] = lengthOfPacketPayload;
   }
   commandBuff[dataOffset++] = lingoID;
   commandBuff[dataOffset++] = commandID;
   commandBuff[dataOffset++] = (transactionID >> 8) & 0xFF;
   commandBuff[dataOffset++] = transactionID & 0xFF;
   bufCopy(commandData, commandBuff, 0, dataOffset, commandDataLen);
   dataOffset += commandDataLen;
   {
      // calculate checksum
      unsigned char checksum = calcChecksum(commandBuff, 1, dataOffset - 1); // minus one to ignore the start byte
      commandBuff[dataOffset] = checksum;
      dataOffset++;
   }
   // return the number of bytes in the commandBuff
   return dataOffset; //plus one for the checksum
}

int transactionCounter = 0;

int generateTransactionID() {
   int retval = transactionCounter;
   transactionCounter++;
   if (transactionCounter > 0xffff) {
      transactionCounter = 0;
   }
   return retval;
}

int IdentifyDeviceLingos(char b[]) 
{
    // Setting all fields to 0xFF displays "Accessory not supported" on Apple device
    char data[12] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
    return AssembleCommand(b, 0x0, 0x13, generateTransactionID(), data, 12);
}

unsigned authentication = 0;
unsigned short lastStartIDPSTransID;

int StartIDPS(char b[]) 
{
    char data[1] = {0xFF};
    authentication++;
    set_iAP_timeout(500*MS); // Timeout before restarting IDPS if we get no response
    lastStartIDPSTransID = generateTransactionID();
    return AssembleCommand(b, 0, GENERAL_LINGO_StartIDPS, lastStartIDPSTransID, data, 0);
}

int GetiPodOptionsForLingo(char b[], char lingo)
{
  char data[1]; data[0] = lingo;
  return AssembleCommand(b, 0x0, GENERAL_LINGO_GetiPodOptionsForLingo, generateTransactionID(), data, 1);
}

int RequestTransportMaxPayloadSize(char b[]) {
  char data[1] = {0xFF}; 
  return AssembleCommand(b, 0, GENERAL_LINGO_RequestTransportMaxPayloadSize, generateTransactionID(), data, 0);
}

int SetEventNotification(char b[]) {
  // Set bits 2 (Flow control) and 5 (Charging info)
  char data[8] = {0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x24};
  return AssembleCommand(b, 0, GENERAL_LINGO_SetEventNotification, generateTransactionID(), data, 8);
}

int SetAvailableCurrent(char b[]) {
  // Set available current to 2100mA
  char data[2] = {0x08, 0x34};
  return AssembleCommand(b, 0, GENERAL_LINGO_SetAvailableCurrent, generateTransactionID(), data, 2);
}

int AccessoryInfoToken(unsigned char b[], unsigned char accInfoType, unsigned char accInfo[], int accInfoLength) {
   // presume that b is big enough 
   b[0] = 3 + accInfoLength;
   b[1] = 0x0; // FIDType
   b[2] = 0x2; // FIDSubtype
   b[3] = accInfoType;
   bufCopy(accInfo, b, 0, 4, accInfoLength);
   return accInfoLength + 4;
}

int SetFIDTokenValues(char b[], int device_id) {
  char data[200];
  int datalen = 0;
  char identTokens[13] = {
                 // IdentifyToken
                 0x0c, // length
                 0x00,0x00, // FIDType, FIDSubtype
                 0x01, // numLingoes
                 0x00, // accessoryLingoes
                 0x00,0x00,0x00,0x02 // DeviceOptions
                 // Space allocated but uninitialised for DeviceID (from auth co-pro)
  };
  char accessoryCapsToken[] = {
                 // AccessoryCapsToken
                 0x0a, // length
                 0x00,0x01, // FIDType, FIDSubtype
                 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 // accCapsBitmask
  };

  // fill in device id in identifyToken
  identTokens[9]  =  device_id & 0xff;
  identTokens[10] = (device_id >> 8) & 0xff;
  identTokens[11] = (device_id >> 16) & 0xff;
  identTokens[12] = (device_id >> 24) & 0xff;

  // num token values
  data[0] = 0x09;
  datalen++;
  // ident tokens
  bufCopy(identTokens, data, 0, datalen, sizeof(identTokens));
  datalen += sizeof(identTokens);
  // accessoryCapsTokens
  bufCopy(accessoryCapsToken, data, 0, datalen, sizeof(accessoryCapsToken));
  datalen += sizeof(accessoryCapsToken);


{
   char accinfobuffer[30];
   int accinfolen;
   char accname[] = VENDOR_STR "USB Audio 2.0"; // Must match USB Product String Descriptor
   char accmanufacturer[] = VENDOR_STR; // Must match USB Manufacturer String Descriptor
   char accfirmwareversion[] = ACCESSORY_FIRMWARE_VERSION;
   char acchardwareversion[] = ACCESSORY_HARDWARE_VERSION;
   char accmodelnumber[] = ACCESSORY_MODEL_NUMBER;
   char accserialnumber[] = SERIAL_STR; // Must match USB Serial Number String Descriptor
   unsigned char accrfcerts[] = {0, 0, 0, 0};

   // Accessory name
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x01, accname, sizeof(accname));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory firmware version
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x04, accfirmwareversion, sizeof(accfirmwareversion));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory hardware version
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x05, acchardwareversion, sizeof(acchardwareversion));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory manufacturer
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x06, accmanufacturer, sizeof(accmanufacturer));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory model number
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x07, accmodelnumber, sizeof(accmodelnumber));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory serial number
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x08, accserialnumber, sizeof(accserialnumber));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;

   // Accessory RF Certifications
   accinfolen =  AccessoryInfoToken(accinfobuffer, 0x0c, accrfcerts, sizeof(accrfcerts));
   bufCopy(accinfobuffer, data, 0, datalen, accinfolen);
   datalen += accinfolen;
  }
  return AssembleCommand(b, 0, GENERAL_LINGO_SetFIDTokenValues, generateTransactionID(), data, datalen);
}

int EndIDPS(char b[], char accEndIDPSStatus) {
  char data[1]; data[0] = accEndIDPSStatus;
  return AssembleCommand(b, 0, GENERAL_LINGO_EndIDPS, generateTransactionID(), data, sizeof(data));
}

int revBytes(int a) {
   int ret  = 0;
   ret |= ((a & 0xFF) << 24);
   ret |= (((a >> 8) & 0xFF) << 16);
   ret |= (((a >> 16) & 0xFF) << 8);
   ret |= ((a >> 24) & 0xFF);
   return ret;
}

// Size to break certificate up into.
//  With this at 499 bytes we overrun the max usb packet size (no room for handshake)
//  At 498 we can use full size USB packets
//  With this at 490 bytes, the max USB packet of iAP will be 503 bytes.
//   Because this is less that the bulk IN endpoint wMaxPacketSize (512) the iPad will continue polling the interrupt IN ep so we can send the sections in a more regular fashion
#define CERT_PAGE_LENGTH 128

int RetAccessoryAuthenticationInfo(char buffer[], unsigned short transactionID, char major, char minor, unsigned short certLen, char cert[], char currentSectionIndex, char maxSectionIndex) 
{
    int sectionLength;
    char data[504]; // This is the command data (from the point of view of the general packet format)
    data[0] = major; // Authentication major version
    data[1] = minor; // Authentication minor version
    data[2] = currentSectionIndex;
    data[3] = maxSectionIndex;
    { 
        // Copy data into data buffer
        int certIndex = 0; // always from the start of the page buffer
        int packetIndex = 4; // Index within the RetAccessoryAuthenticationInfo command data
        if (currentSectionIndex != maxSectionIndex) 
        {
            // Not last section
            sectionLength = CERT_PAGE_LENGTH;
        } 
        else 
        {
            // Last section
            sectionLength = certLen % CERT_PAGE_LENGTH;
            if (sectionLength == 0) 
            {
                sectionLength = CERT_PAGE_LENGTH;
            }
        }
        bufCopy(cert, data, certIndex, packetIndex, sectionLength);
    }
    //assert(length <= 512); // size of buffer
    return AssembleCommand(buffer, 0, GENERAL_LINGO_RetAccessoryAuthenticationInfo, transactionID, data, 4 + sectionLength); // return length of buffer
}

int RetAccessoryAuthenticationSignature(char buffer[], unsigned short transactionID, unsigned signLen, char sign[]) {
   return AssembleCommand(buffer, 0, GENERAL_LINGO_RetAccessoryAuthenticationSignature, transactionID, sign, signLen);
}

// Adds to code size if there is any printf in the program
//void printBuf(char buf[], int len) {
//   for(int i = 0; i != len; i++) {
//      iprintf("0x%02x, ", buf[i]);
//   }
//   iprintf("\n");
//}

void checkERROR_CODE(chanend ?c_i2c, char string[], port ?p_scl, port? p_sda) 
{
    unsigned char errorCode[1];
    unsigned char authControl[1];
    iic_readC(0x11, 0x10, authControl, 1, c_i2c, p_scl, p_sda);
    if (authControl[0] & 0x80) 
    { // ERR_SET set
        iic_readC(0x11, 0x05, errorCode, 1, c_i2c, p_scl, p_sda); // Also clear error code
        if (errorCode[0] != 0) 
        {
            //iprintf("ERROR: Error code is: 0x%02x at %s\n", errorCode[0], string);
        }
    }
}

// state for iAP
char iap_buffer[IAP_BUFFER_SIZE];
int iap_bufferlen;
int data_to_send = 0;

// state for GetAccessoryAuthenticationSignature
unsigned authenticating = 0;

// state for iOS supported features
int iOSSupportsUsbAudioOutput = 0;
int iOSSupportsUsbAudioInput = 0;

int parseIdentification(unsigned char commandID, int bufferpos, unsigned short transID, chanend ?c_i2c, 
    port ?p_scl, port ?p_sda) 
{
   switch(commandID) {
      case GENERAL_LINGO_StartIDPS: // ACK
      {
         unsigned char ackResultStatus = iap_buffer[bufferpos-2];
         // Ensure that the ACK is for the latest StartIDPS sent
         if (transID == lastStartIDPSTransID)
         {
            clear_iAP_timeout();
            switch(ackResultStatus) {
               case IPOD_ACK_Success:
                  iap_bufferlen = GetiPodOptionsForLingo(iap_buffer, 0x0);
                  data_to_send = 1;
                  break;
               case IPOD_ACK_Bad_parameter: // IDPS not supported by Apple device
                  // We only support identification using IDPS
                  // Send IDL with val 0xFF to display "Accessory not supported" message
                  iap_bufferlen = IdentifyDeviceLingos(iap_buffer);
                  data_to_send = 1;
                  break;
               case IPOD_ACK_Maximum_connections_reached:
               default:
                  // Must not attempt any further iAP commands until it has been disconnected
                  break;
            }
         }
         break;
      }
      case GENERAL_LINGO_RetiPodOptionsForLingo:
      {
         unsigned char lingoID = iap_buffer[bufferpos++];
         unsigned char optionBits[8];
         
         //printstr("RetiPodOptionsForLingo, lingoID: ");
         //printhexln(lingoID);
         //printstrln("RetiPodOptionsForLingo, optionBits: ");
         for (int i = 0; i < 8; i++)
         {
            optionBits[i] = iap_buffer[bufferpos++];
            //printhexln(optionBits[i]);
         }
         
         // Check required options are supported
         switch (lingoID)
         {
            case GENERAL_LINGO:
               // bit27 - host mode invoked by ID resistor supported
               if (!(optionBits[4] & 0x8))
               {
                  //printstrln("Invoking host mode using ID resistor not supported");
               }
               // bit28 - host mode supports output to USB audio devices (must also have bit27 set)
               if (optionBits[4] & 0x18)
               {
                  iOSSupportsUsbAudioOutput = 1;
                  //printstr("iOSSupportsUsbAudioOutput");
               }
               // bit29 - host mode supports input from USB audio devices (must also have bit27 set)
               if (optionBits[4] & 0x28)
               {
                  iOSSupportsUsbAudioInput = 1;
                  //printstr("iOSSupportsUsbAudioInput");
               }
               break;
            default: // Unhandled lingo
               //printstr("RetiPodOptionsForLingo for unhandled lingo received, lingoID: ");
               //printhexln(lingoID);
               break;
         }
         iap_bufferlen = RequestTransportMaxPayloadSize(iap_buffer);
         data_to_send = 1;
         break;
      }
      case GENERAL_LINGO_ReturnTransportMaxPayloadSize:
      {
         int device_id = 0xbaddbabe;
         unsigned int maxPayload = iap_buffer[bufferpos++];
         int retval = 1;
         int attempts = 0;
         maxPayload = maxPayload << 8 | iap_buffer[bufferpos++];
         //printstr("Max payload is: ");
         //printintln(maxPayload);
         // Get device ID, required for SetFIDTokenValues command
         do 
         {
            retval = coProcessorGetDeviceID(device_id, c_i2c, p_scl, p_sda);
            attempts++;
         } 
         while((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         
         if (attempts == COPROCESSOR_RETRIES) 
         {
            printstrln("Failed to get device id");
         }
         iap_bufferlen = SetFIDTokenValues(iap_buffer, device_id);
         data_to_send = 1;
         break;
      }
      case GENERAL_LINGO_AckFIDTokenValues:
      {
         //printstrln("Got AckFIDTokenValues");
         // Check all tokens accepted
         int successes = 0;
         int failed = 0;
         int restartRequired = 0;
         char numFIDTokenValueACKs = iap_buffer[bufferpos++];
         for (int i = 0; i < numFIDTokenValueACKs; i++)
         {
            char length = iap_buffer[bufferpos++];
            char FIDType = iap_buffer[bufferpos++];
            char FIDSubtype = iap_buffer[bufferpos++];
            char ACKStatus = iap_buffer[bufferpos++];
            // Advance the buffer position past any remaining parts of this tokens ACK
            switch (FIDSubtype) {
               case 0x00: // IdentifyToken
                  if (length - 3) bufferpos+=(length - 3); // 3 bytes for FID type, subtype, and ACK status
                  break;
               case 0x02: // AccessoryInfoToken - 1 byte for accInfoType
               case 0x03: // iPodPreferenceToken - 1 byte for iPodPrefClass
               case 0x04: // EAProtocolToken - 1 byte for protocolIndex
                  bufferpos++;
                  break;
               /*
               case 0x01: // AccessoryCapsToken
               case 0x05: // BundleSeedPrefToken
               case 0x07: // ScreenInfoToken
               case 0x08: // EAProtocolMetadataToken
               case 0x0E: // AccessoryDigitalAudioSampleRatesToken
               case 0x0F: // AccessoryDigitalAudioVideoDelayToken
               */
               default: // No extra fields to skip over
                  break;
            }
            
            switch (ACKStatus) {
               case 0: // Token accepted
                  successes++;
                  break;
               case 1: // Required token-value field failed
               case 2: // Optional token-value field recognised, but failed
               {
                  // Retry sending FIDTokenValues
                  int device_id = 0xbaddbabe;
                  int retval = 1;
                  int attempts = 0;
                  
                  /* if we could control which tokens SetFIDTokenValues() sent:
                  switch (FIDSubtype) {
                     case 0x00: // IdentifyToken
                        break;
                     case 0x01: // AccessoryCapsToken
                        break;
                     case 0x02: // AccessoryInfoToken
                        break;
                     default: // Unhandled token type
                        break;
                  }*/
                  
                  // Get device ID, required for SetFIDTokenValues command
                  do 
                  {
                     retval = coProcessorGetDeviceID(device_id, c_i2c, p_scl, p_sda);
                     attempts++;
                  } 
                  while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
                  
                  if (attempts == COPROCESSOR_RETRIES) 
                  {
                     printstrln("Failed to get device ID");
                  }
                  iap_bufferlen = SetFIDTokenValues(iap_buffer, device_id);
                  data_to_send = 1;
                  break;
               }
               case 3: // Token not supported, must not retry
                  failed++;
                  break;
               case 4: // Lingoes busy
               case 5: // Maximum connections reached
               default:
                  restartRequired++; // Retry entire IDPS process
                  break;
            }
         }
         if (successes == numFIDTokenValueACKs)
         {
            // EndIDPS requesting to start authentication
            iap_bufferlen = EndIDPS(iap_buffer, 0);
            data_to_send = 1;
         }
         else if (failed)
         {
            // EndIDPS without continuing to auth, abandoning IDPS process
            iap_bufferlen = EndIDPS(iap_buffer, 2);
            data_to_send = 1;
         }
         else if (restartRequired)
         {
            // EndIDPS resetting all IDPS information
            iap_bufferlen = EndIDPS(iap_buffer, 1);
            data_to_send = 1;
            set_iAP_timeout(0); // Retry StartIDPS immediately
         }
         // Else wait for AckFIDTokenValues of retried tokens
         break;
      }
      case GENERAL_LINGO_IDPSStatus:
         //printstrln("Got IDPSStatus");
         // Check IDPS status
         if (iap_buffer[bufferpos++] != 0x00)
         {
            // An error occured during Identification, setup timeout to retry
            set_iAP_timeout(500*MS); // Timeout before restarting IDPS
         }
         break;
      default:
         return -1; // Unhandled command
         break;
   }
   return 0;
}

// state for GetAccessoryAuthenticationInfo (global because returned on several RetAccessoryAuthenticationInfos)
char major = 0xf;
char minor = 0xb;
char cert[CERT_PAGE_LENGTH];
unsigned short certLen = 0;
int currentSectionIndex;
int maxSectionIndex;

int parseAuthentication(unsigned char commandID, int bufferpos, unsigned short transID, chanend ?c_i2c,
    port ?p_scl, port ?p_sda) 
{
    switch(commandID) 
    {
        case GENERAL_LINGO_GetAccessoryAuthenticationInfo:
        {
            int requiredSections;
            int retval = 1;
            int attempts = 0;
            // Clear any outstanding timeouts
            clear_iAP_timeout();
         
            // Handle sending certificate
            do 
            {
                retval = coProcessorGetAuthVersion(major, minor, c_i2c, p_scl, p_sda);
                attempts++;
            } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         
            if (attempts == COPROCESSOR_RETRIES) 
            {
                //printstrln("Failed to get auth version");
            }
 
            do 
            {
                retval = coProcessorGetAccessoryCertificateLength(certLen, c_i2c, p_scl, p_sda);
                attempts++;
            } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         
            if (attempts == COPROCESSOR_RETRIES) 
            {
                //printstrln("Failed to get accessory certificate length");
            }
         
            requiredSections = certLen / CERT_PAGE_LENGTH; // 500 bytes being the maximum per RetAccessoryAuthenticationInfoCommand
            if (certLen % CERT_PAGE_LENGTH) 
            { 
                requiredSections++; 
            }
            maxSectionIndex = requiredSections - 1;
         
            currentSectionIndex = 0;
            do 
            {
                retval = coProcessorGetAccessoryCertificatePage(CERT_PAGE_LENGTH, cert, currentSectionIndex, c_i2c, p_scl, p_sda);
                attempts++;
            } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
            if (attempts == COPROCESSOR_RETRIES) 
            {
                //printstrln("Failed to get accessory certificate page");
            }
            iap_bufferlen = RetAccessoryAuthenticationInfo(iap_buffer, transID, major, minor, certLen, cert, currentSectionIndex, maxSectionIndex);
            data_to_send = 1;
            //currentSectionIndex++;
            break;
        }
        case GENERAL_LINGO_RetAccessoryAuthenticationInfo: // ACK
        {
            unsigned char ackResultStatus = iap_buffer[bufferpos-2];
            int retval = 1;
            int attempts = 0;
            switch(ackResultStatus) {
            case IPOD_ACK_Success:
               currentSectionIndex++;
               break;
            default: // Error - resend the last page
               break;
         }
         // cert, major and minor should be set up by handling of GetAccessoryAuthenticationInfo
         do {
            retval = coProcessorGetAccessoryCertificatePage(CERT_PAGE_LENGTH, cert, currentSectionIndex, c_i2c, p_scl, p_sda);
            attempts++;
         } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         if (attempts == COPROCESSOR_RETRIES) {
            //printstrln("Failed to get accessory certificate page");
         }
         iap_bufferlen = RetAccessoryAuthenticationInfo(iap_buffer, transID, major, minor, certLen, cert, currentSectionIndex, maxSectionIndex);
         data_to_send = 1;
         break;
      }
      case GENERAL_LINGO_AckAccessoryAuthenticationInfo:
         // Check error code here
         switch (iap_buffer[bufferpos++]) {
            case 0x00: // Authentication information supported
               // Nothing to do now, Apple device should send GetAccessoryAuthenticationSignature next
               break;
            case 0x04: // Certificate too long or sections out of order
            case 0x08: // Authentication information unsupported
            case 0x0A: // Certificate invalid
            case 0x0B: // Certificate permissions are invalid
               // Error
               set_iAP_timeout(5000*MS); // Timeout before restarting IDPS
               break;
         }
         break;
      case GENERAL_LINGO_GetAccessoryAuthenticationSignature:
      {
         char challenge[20];
         int retval = 1;
         int attempts = 0;
         // Clear any outstanding timeouts
         clear_iAP_timeout();
         
         bufCopy(iap_buffer, challenge, 6, 0, 20); // get challenge from GetAccessoryAuthenticationSignature
         do {
            retval = coProcessorPutChallenge(20, challenge, c_i2c, p_scl, p_sda);
            attempts++;
         } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         if (attempts == COPROCESSOR_RETRIES) {
            //iprintf("coProcessorPutChallenge failed at stage %d\n", retval);
         }
         
         do {
            retval = coProcessorStartSigGeneration(c_i2c, p_scl, p_sda);
            attempts++;
         } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
         if (attempts == COPROCESSOR_RETRIES) {
            //printstrln("coProcessorStartSigGeneration failed");
         }
         
         {
            char signature[128];// = {0xBB, 0xAA, 0xDD, 0xDD, 0xFF, 0x00, 0x00, 0xDD};
            unsigned signatureLength = 8 ;
            do {
               retval = coProcessorGetSignature(signatureLength, signature, c_i2c, p_scl, p_sda);
               attempts++;
            } while ((retval != 0) && (attempts < COPROCESSOR_RETRIES));
            if (attempts == COPROCESSOR_RETRIES) {
               //printstrln("coProcessorGetSignature failed");
            }
            
            iap_bufferlen = RetAccessoryAuthenticationSignature(iap_buffer, transID, signatureLength, signature);
         }
         data_to_send = 1;
         break;
      }
      case GENERAL_LINGO_AckAccessoryAuthenticationStatus:
         // Check return code
         if (iap_buffer[bufferpos++] == 0x00) // Authentication operation passed
         {
            iap_bufferlen = SetAvailableCurrent(iap_buffer);
            //iap_bufferlen = SetEventNotification(iap_buffer);
            data_to_send = 1;
         } 
         else
         {
            // Any other return code means authentication operation failed
            set_iAP_timeout(5000*MS); // Timeout before restarting IDPS
         }
         break;
      default:
         return -1; // Unhandled command
         break;
   }
   return 0;
}

int parseiAP(chanend ?c_i2c, port ?p_scl, port ?p_sda) 
{
    int bufferpos = 0;
    unsigned char startOfPacketMarker = iap_buffer[bufferpos++];
    unsigned char lengthOfPacketPayload = iap_buffer[bufferpos++];
    unsigned char lingoID = iap_buffer[bufferpos++];
    unsigned char commandID = iap_buffer[bufferpos++];
    unsigned int transID = iap_buffer[bufferpos++];
    
    transID = transID << 8 | iap_buffer[bufferpos++];

    //printstrln("parseiAP");

    if (startOfPacketMarker != 0x55) 
    {
      return -1;
    }
   
    if (lengthOfPacketPayload == 0x0) 
    {
        // Never expect to get long packet format
        return -1;
    }

    switch(lingoID) 
    {
        case GENERAL_LINGO:
            switch(commandID) 
            {
            
                case GENERAL_LINGO_RequestIdentify:
                    iap_bufferlen = StartIDPS(iap_buffer);
                    data_to_send = 1;
                    authenticating = 1;
                    // Clear any outstanding timeouts
                    clear_iAP_timeout();
                    break;
            
                case GENERAL_LINGO_iPodAck:
                {
                    //unsigned char resultStatus = iap_buffer[bufferpos++];
                    unsigned char ackedCommandId;
                    bufferpos++; // skip resultStatus
                    ackedCommandId = iap_buffer[bufferpos++];
                    switch(ackedCommandId) 
                    {
                        case GENERAL_LINGO_StartIDPS:
                            parseIdentification(ackedCommandId, bufferpos, transID, c_i2c, p_scl, p_sda);
                            break;
                        case GENERAL_LINGO_RetAccessoryAuthenticationInfo:
                            parseAuthentication(ackedCommandId, bufferpos, transID, c_i2c, p_scl, p_sda);
                            break;
                        case GENERAL_LINGO_SetAvailableCurrent:
                            authenticating = 0; // Change guard for non iAP cases
                            
                            /* We can now disable the CoProcessor */
                            CoProcessorDisable();

                            break;
                         default:
                            break;
                    }
                    break;
                }
            
                // Identification
                case GENERAL_LINGO_RetiPodOptionsForLingo:
                case GENERAL_LINGO_ReturnTransportMaxPayloadSize:
                case GENERAL_LINGO_AckFIDTokenValues:
                case GENERAL_LINGO_IDPSStatus:
                    parseIdentification(commandID, bufferpos, transID, c_i2c, p_scl, p_sda);
                    break;
            
                // Authentication
                case GENERAL_LINGO_GetAccessoryAuthenticationInfo:
                case GENERAL_LINGO_AckAccessoryAuthenticationInfo:
                case GENERAL_LINGO_GetAccessoryAuthenticationSignature:
                case GENERAL_LINGO_AckAccessoryAuthenticationStatus:
                    parseAuthentication(commandID, bufferpos, transID, c_i2c, p_scl, p_sda);
                    break;

                default: // Unhandled command in general lingo
                    return -1;
            }
            break;
        default: // Unhandled lingo
            return -1;
    }
    return 0;
}

int ith_count = 0; // Count of things sent to host
// Buffers
queue to_host_fifo;
unsigned char to_host_arr[256];
queue from_host_fifo;
unsigned char from_host_arr[256];
// State
unsigned expecting_length = 1; // Expecting the next data item to be a length
unsigned expected_data_length = 0; // The length of data we are expecting

void init_iAP(chanend ?c_i2c, port ?p_scl, port ?p_sda) 
{
    // iAP initialisation
    // Initialise buffers
    init_queue(to_host_fifo, to_host_arr, 256, 1);
    init_queue(from_host_fifo, from_host_arr, 256, 1);
    // Initialise coprocessor
    coProcessorInit(iAPTimer, c_i2c, p_scl, p_sda);
}

void handle_iap_case(int is_ack, int is_reset, unsigned int datum, chanend c_iap, chanend ?c_i2c,
    port ?p_scl, port ?p_sda) 
{
    if (is_reset) 
    {
        /* Prevent multiple issuing of StartIDPS (may need to handle it getting lost) */
        unsigned device_detect = GetIDeviceDetect();
        
        if (!device_detect) 
        {
            // Apple device detected so startIDPS
            // Reset transaction ID counter each time a new device is detected
            transactionCounter = 0;
            // Reset iAP retry/timeout state
            clear_iAP_timeout();
            // Start buffer with StartIDPS message in
            iap_bufferlen = StartIDPS(iap_buffer);
            for (int i = 0; i != iap_bufferlen; i++) 
            {
                enqueue(to_host_fifo, iap_buffer[i]);
            }
            //printstrln("Queued");
            //dump(to_host_fifo);
            // Start the ball rolling (so I will be expecting an ack)
            outuint(c_iap, iap_bufferlen);
            expecting_length = 1; //Really expecting ack but if it's data then it'll be a length
            authenticating = 1;
        
            /* We want to authenticate - enable CoProcessor */
            CoProcessorEnable();
        } 
        else 
        {
            /* No Apple device, disable co-processor */
            CoProcessorDisable();
        }
     } 
     else 
     {
        //printstrln("iap_get_ack_or_data");
        if (is_ack) 
        {
            // have we got more data to send
          //  printstr("ack\n");
            if (!isempty(to_host_fifo)) 
            {
                //printstr("iap->decouple\n");
                outuint(c_iap, dequeue(to_host_fifo));
                ith_count++;
            } 
        } 
        else 
        {
            //printstr("data\n");
            if (expecting_length) 
            {
                expected_data_length = datum;
                expecting_length = 0;
                // iap_send_ack(c_iap); // Don't send ack as I don't expect it at the other end!
             } 
             else 
             {
                // Expecting data
                int fullness;
                enqueue(from_host_fifo, datum);
                iap_send_ack(c_iap);
                fullness = items(from_host_fifo);
                if (fullness == expected_data_length) 
                {
                    // Received whole message. Transfer to iap_buffer for parse
                    for (int i = 0; i != fullness; i++) 
                    {
                        iap_buffer[i] = dequeue(from_host_fifo);
                    }
                    iap_bufferlen = expected_data_length;
                    // Parse iAP from host
                    
                    parseiAP(c_i2c, p_scl, p_sda);
                    // Start the ball rolling
                    if (data_to_send) 
                    {
                        for (int i = 0; i != iap_bufferlen; i++) 
                        {
                            enqueue(to_host_fifo, iap_buffer[i]);
                            //printhexln(dequeue(from_host_fifo));
                        }
                        outuint(c_iap, iap_bufferlen);
                        data_to_send = 0; // In this usage data_to_send is just used to kick off the data. The rest is pulled by ACKs.
                    }
                    expecting_length = 1;
                }
            }
        }
    }
}

unsigned polltime;
int previous_device_detect = 1; /* Not detected */

/* Read iDevice detect - returns 1 for change, 0 else */
int handle_poll_dev_det(timer poll) 
{
    int retval = 0;
    unsigned device_detect = GetIDeviceDetect();
    
    if (device_detect != previous_device_detect) 
    {

        if (device_detect) 
        { 
            // high: USB B
            SelectUSBPc();
        }
        else 
        { 
            // low: dock
            SelectUSBDock();
        }
        previous_device_detect = device_detect;
        retval = 1;
    }
    polltime += XS1_TIMER_HZ;// / 2;

    return retval;
}

unsigned int iAP_state_timeout;
int iAP_state_retriesRemaining;

void set_iAP_timeout(unsigned int timeout) 
{
    // Setup timeout
    iAPTimer :> iAP_state_timeout;
    iAP_state_timeout+= timeout;
    iAP_state_retriesRemaining = 1;
}

void clear_iAP_timeout() 
{
    // Clear retry counter, then timeout actions will not be taken
    iAP_state_retriesRemaining = 0;
}

void check_iAP_timeout(chanend c_iap) 
{       
    if (iAP_state_retriesRemaining)
    {
        select 
        {
            case iAPTimer when timerafter(iAP_state_timeout) :> void:
                iAP_state_retriesRemaining--;
                iap_bufferlen = StartIDPS(iap_buffer);
                for (int i = 0; i != iap_bufferlen; i++) 
                {
                    enqueue(to_host_fifo, iap_buffer[i]);
                }
                outuint(c_iap, iap_bufferlen);
                authenticating = 1;
                break;
            default: // Not timed out yet
                break;
        }
    }
}

/* Stand-alone iAP thread - for the case when MIDI is not used */
void iAP(chanend c_iap, chanend ?c_i2c, port ?p_scl, port ?p_sda) 
{
    iAPTimer :> polltime;
    polltime += XS1_TIMER_HZ / 2;
    init_iAP(c_i2c, p_scl, p_sda);
    
    while (1) 
    {
        int is_ack;
        int is_reset;
        unsigned int datum;
        select 
        {
            case iap_get_ack_or_reset_or_data(c_iap, is_ack, is_reset, datum):
                handle_iap_case(is_ack, is_reset, datum, c_iap, c_i2c, p_scl, p_sda);
                break;

            case iAPTimer when timerafter(polltime) :> void:
                if(!handle_poll_dev_det(iAPTimer)) // Device detect poll interval determines smallest iAP timeout
                {
                    check_iAP_timeout(c_iap);
                }
                break;
        }
    }
}
