/*
 * board.c
 *
 * Created: 20. 1. 2014 12:18:59
 *  Author: Jaroslav Dohnal
 *
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "board.h"

/************************************************************************/
/*				            	DEFINES                                 */
/************************************************************************/

void board_init(void){
	
	//everything as output
	F_SEL48_DDR |= (1<<F_SEL48_PIN);
	F_SEL44_DDR |= (1<<F_SEL44_PIN);
	
	_RST_DDR |= (1<<_RST_PIN);
	
	M0_DDR |= (1<<M0_PIN);
	M1_DDR |= (1<<M1_PIN);
	
	_CS_LEFT_DDR |= (1<<_CS_LEFT_PIN);
	_CS_RIGHT_DDR |= (1<<_CS_RIGHT_PIN);
	CSCK_DDR |= (1<<CSCK_PIN);
	CDATA_DDR |= (1<<CDATA_PIN);
	
	board_setSampleFreq(48); //initialize board at 48 kHz fs
}

void board_setSampleFreq(uint8_t freq){ //set sample frequency of the board
	
	uint8_t OverSampingRatio;
	
	board_reset(TRUE);
	
	if (freq % 44 == 0)  //set MCK oscillator
		setOscillator(44);
	
	else
		setOscillator(48);
	
	_delay_ms(1);
	board_reset(FALSE);
	DAC_init();
	
	OverSampingRatio = (freq / 44) - 1;
	if (OverSampingRatio >= 3)
		OverSampingRatio = 2; //calculate oversampling ratio 
	
	DAC_OSR(OverSampingRatio);
	ADC_OSR(OverSampingRatio);
}

void board_reset(uint8_t status){
	
	if (status == FALSE)
		_RST_PORT |= (1<<_RST_PIN);
	else
		_RST_PORT &= ~(1<<_RST_PIN);
}

void setOscillator(uint8_t fs){
	
	if (fs == 48)
	{
		F_SEL48_PORT |= (1<<F_SEL48_PIN);
		F_SEL44_PORT &= ~(1<<F_SEL44_PIN);
	} 
	else
	{	
		F_SEL44_PORT |= (1<<F_SEL44_PIN);
		F_SEL48_PORT &= ~(1<<F_SEL48_PIN);
	}
}

void DAC_init(void){
	
	controlSPI(0x201, LEFT); //see datasheet of the AD1955
	controlSPI(0x201, RIGHT);
}

void DAC_OSR(uint16_t ratio){
	
	ratio = (ratio << 8);
	controlSPI((ratio | 0x800), LEFT); //see datasheet of the AD1955
	controlSPI((ratio | 0xC00), RIGHT);
}

void ADC_OSR(uint8_t ratio){
	
	M0_PORT |= (ratio << M0_PIN);
	M1_PORT |= (ratio << (M1_PIN - 1));
}

void controlSPI(uint16_t data, uint8_t chanel){
	
	int i;
	
	if (chanel == LEFT)	// chip select
		_CS_LEFT_PORT &= ~(1<<_CS_LEFT_PIN);
	else
		_CS_RIGHT_PORT &= ~(1<<_CS_RIGHT_PIN);
	_delay_us(SPI_delay);
	
	for (i=15; i>=0; i--)
	{
		CSCK_PORT &= ~(1<<CSCK_PIN); //clock low
		_delay_us(SPI_delay);
		
		if (data & 1<<i)
		CDATA_PORT |= (1<<CDATA_PIN);  //data out
		else
		CDATA_PORT &= ~(1<<CDATA_PIN);
		
		_delay_us(SPI_delay);
		CSCK_PORT |= (1<<CSCK_PIN); //clock pulse
		_delay_us(SPI_delay);
	}

	 //chip unslect
	_CS_LEFT_PORT |= (1<<_CS_LEFT_PIN);
	_CS_RIGHT_PORT |= (1<<_CS_RIGHT_PIN);
}
