/*
 * board.h
 *
 * Created: 20. 1. 2014 12:18:49
 *  Author: Jaroslav Dohnal
 *
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef BOARD_H_
#define BOARD_H_

#ifndef F_CPU
#define F_CPU 8000000UL
#endif

/************************************************************************/
/*				            	INCLUDES                                */
/************************************************************************/

#include <util/delay.h>
#include <avr/io.h>

/************************************************************************/
/*				            	DEFINES                                 */
/************************************************************************/

#define TRUE				1
#define FALSE				0

//DAC chanels
#define LEFT				0
#define RIGHT				1

//oscilators pin defines
#define F_SEL44_PIN			PC3
#define F_SEL44_PORT		PORTC
#define F_SEL44_DDR			DDRC
#define F_SEL48_PIN			PC2
#define F_SEL48_PORT		PORTC
#define F_SEL48_DDR			DDRC

//reset pin define
#define _RST_PIN			PC4
#define _RST_PORT			PORTC
#define _RST_DDR			DDRC

//ADC pin defines
#define	M1_PIN				PC5
#define	M1_PORT				PORTC
#define	M1_DDR				DDRC		
#define M0_PIN				PD0
#define M0_PORT				PORTD
#define M0_DDR				DDRD

//control SPI pin defines
#define _CS_RIGHT_PIN		PD1
#define _CS_RIGHT_PORT		PORTD
#define _CS_RIGHT_DDR		DDRD
#define CSCK_PIN			PD2
#define CSCK_PORT			PORTD
#define CSCK_DDR			DDRD
#define	CDATA_PIN			PD3
#define	CDATA_PORT			PORTD
#define	CDATA_DDR			DDRD
#define _CS_LEFT_PIN		PD4
#define _CS_LEFT_PORT		PORTD
#define _CS_LEFT_DDR		DDRD

//control SPI delay define in us
#define SPI_delay			100

/************************************************************************/
/*				            	DECLARATIONS                            */
/************************************************************************/

void board_init(void); //initialize the board
void board_setSampleFreq(uint8_t freq); //sets board to the desired sample rate

//internal functions
void board_reset(uint8_t status);
void setOscillator(uint8_t fs);
void DAC_init(void);
void DAC_OSR(uint16_t ratio);
void ADC_OSR(uint8_t ratio);
void controlSPI(uint16_t data, uint8_t chanel);

#endif /* BOARD_H_ */